# -*- coding: utf-8 -*-

import re
import subprocess

import click

REGEXP = r'https://.+&bucket=(?P<bucket>.*)&prefix=(?P<prefix>.*)'
S3URI_FORMAT = 's3://{bucket}/{prefix}'


def url2uri(url):
    """
    convert URL to s3 uri

    - Bucket root
    "https://console.aws.amazon.com/s3/home?region=<your_region>#&bucket=mybucket&prefix="
    ->
    "s3://mybucket/"

    - Top layer directory
    "https://console.aws.amazon.com/s3/home?region=<your_region>#&bucket=mybucket&prefix=mydir/"
    ->
    "s3://mybucket/mydir/"

    - Nested directory
    "https://console.aws.amazon.com/s3/home?region=<your_region>#&bucket=mybucket&prefix=mydir/nested/"
    ->
    "s3://mybucket/mydir/nested/"
    """
    match = re.match(REGEXP, url)
    if not match:  # does not match S3 url
        return url
    groupdict = match.groupdict()  # {'bucket': ..., 'prefix': ...}
    s3uri = S3URI_FORMAT.format(**groupdict)
    return s3uri


@click.command(context_settings={'ignore_unknown_options': True})
@click.argument('subcommand', type=click.Choice(['ls', 'rm', 'cp', 'mv', 'sync', 'presign']))
@click.argument('args', nargs=-1)
def aws_s3_url2uri(subcommand, args):
    """Entry point"""
    uried_args = [url2uri(arg) for arg in args]
    command = ['aws', 's3', subcommand] + uried_args
    print(' '.join(command))
    _ = subprocess.call(command)



# coding=utf-8

import unittest

from aws_s3_url2uri.main import url2uri


class TestURL2URI(unittest.TestCase):
    def test_bucket_root(self):
        """bucket root URL shold be converted to URI"""
        url = 'https://console.aws.amazon.com/s3/home?region=<your_region>#&bucket=mybucket&prefix='
        actual = url2uri(url)
        expected = 's3://mybucket/'
        self.assertEqual(actual, expected)

    def test_top_layer_directory(self):
        """top layer directory shold be converted to URI"""
        url = 'https://console.aws.amazon.com/s3/home?region=<your_region>#&bucket=mybucket&prefix=mydir/'
        actual = url2uri(url)
        expected = 's3://mybucket/mydir/'
        self.assertEqual(actual, expected)

    def test_nested_directory(self):
        """nested directory shold be converted to URI"""
        url = 'https://console.aws.amazon.com/s3/home?region=<your_region>#&bucket=mybucket&prefix=mydir/nested/'
        actual = url2uri(url)
        expected = 's3://mybucket/mydir/nested/'
        self.assertEqual(actual, expected)

    def test_non_url_untouched(self):
        """If parameter does not start with 'https://' it shold not be converted"""
        non_url = '/home/myname/tmp/dir'
        actual = url2uri(non_url)
        expected = non_url
        self.assertEqual(actual, expected)

    def test_unicode_non_url_untouched(self):
        """Usually your local system allows unicode path"""
        unicode_non_url = 'ホーム/名前/dir'
        actual = url2uri(unicode_non_url)
        expected = unicode_non_url
        self.assertEqual(actual, expected)



# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
from pip.req import parse_requirements

with open('README.md') as f:
    long_description = f.read()

install_requires = [
    str(pr.req) for pr in parse_requirements('requirements.txt', session='hack')
]


setup(
    name='aws_s3_url2uri',
    description='Make awscli s3 commands work with URL',
    long_description=long_description,
    version='0.1.3',
    author='NAKATA Keisuke',
    author_email='keisuke.nakata.919@gmail.com',
    url='https://github.com/keisuke-nakata/aws-cli-s3-url2uri',
    license='Apache License 2.0',
    packages=find_packages(),
    install_requires=install_requires,
    test_suite='tests',
    entry_points={
        'console_scripts': [
            'aws_s3_url2uri=aws_s3_url2uri.main:aws_s3_url2uri'
        ]
    }
)
